/*
 * DiaSCE is a code editor for C and C++.
 * Copyright (C) 2000  Ander Lozano Prez
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Ander Lozano Prez
 * c/Juan de Gardeazabal 4, 1 D
 * 48004 Bilbao
 * Vizcaya
 * Spain
 *
 * ander1@wanadoo.es
 */

#include "main.h"

//*******************************************************************

void gen_salir(void)
{
	edit_cerrar();
	gla_free_glade_window_list();
	pref_cerrar();
	xmlFreeDoc(proyecto.xml);
	g_free(proyecto.archivo);
	if (texto_busqueda_actual != NULL)
		g_free(texto_busqueda_actual);
	g_free(pro_proyecto_cargar);
	gtk_main_quit();
}

void gen_cerrar_ventana(GtkWidget *widget)
{
	GtkWidget *ventana;
	
	ventana=gtk_widget_get_toplevel(widget);
	gtk_widget_hide(ventana);
}

void gen_anadir(GtkWidget *widget)
{
	GtkTreeIter nodo_arbol;
	GtkWidget *ventana;
	gchar **seleccion;
	gchar *archivo;
	guint cont;
	gchar *destino;
	
	ventana=gtk_widget_get_toplevel(widget);
	seleccion=gtk_file_selection_get_selections(GTK_FILE_SELECTION(ventana));
	archivo=seleccion[0];
	if (archivo[0]==0) {
		g_strfreev(seleccion);
		return;
	}	 
	for (cont=0;archivo[cont]!=0;cont++);
	if (archivo[cont-1]=='/') {
		g_strfreev(seleccion);
		return;
	}
	for (;archivo[cont]!='/';cont--);
	cont++;
	
	if (!edit_repetido(archivo+cont)) {
		arch_anadir(archivo+cont,&nodo_arbol);
		edit_anadir(archivo,&nodo_arbol);
		pro_anadir(archivo+cont);
		pro_guardar();
		edit_ver(archivo+cont);
		destino=pro_nombre_completo_archivo(archivo+cont,FALSE, UTF8);
		if (strcmp(destino,archivo)) {
			gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(edit_actual->datos),TRUE);
			edit_guardar();
		}
		g_free(destino);
	} else {
		ventana=glade_xml_get_widget(repe_ventana,"repe_ventana");
		gtk_widget_show(ventana);
	}
	g_strfreev(seleccion);
}

guint gen_comando(gchar *comando, gchar *mensaje)
{
	FILE *tuberia;
	guint salida;
	gchar letra;
	gchar *cadena;
	gchar *temporal;
	guint mas;
	gsize bytes_read, bytes_written;
	GError *error;
	gboolean result =FALSE;
	const gchar *punto_malo;
	G_CONST_RETURN char *locale;
	
	DEBUG_MSG(->gen_comando);
	gen_limpiar_mensajes();
	if (mensaje!=NULL) {
		g_print("%s",mensaje);
	} else {
		g_print("%s",comando);
	}
	tuberia=popen(comando,"r");
	temporal=NULL;
	cadena=NULL;
	do {
		gen_refrescar();
		mas=fread(&letra,1,1,tuberia);
		if (mas) {
			if (temporal!=NULL) {
				cadena=g_strdup_printf("%s%c",temporal,letra);
				g_free(temporal);
				temporal=cadena;
			} else {
				cadena=g_strdup_printf("%c",letra);
				temporal=cadena;
			}
			if (letra=='\n') {
				if (gen_es_salida_grep(cadena)) {
					result = g_utf8_validate(temporal, -1, &punto_malo);
					if (!result) {
						temporal = cadena;
						g_get_charset(&locale);
						cadena = g_convert(temporal, strlen(temporal), "UTF-8", locale, &bytes_read, &bytes_written, &error);
						g_free(temporal);
					}
					g_printerr("%s",cadena);
				} else {
					result = g_utf8_validate(temporal, -1, &punto_malo);
					if (!result) {
						temporal = cadena;
						g_get_charset(&locale);
						cadena = g_convert(temporal, strlen(temporal), "UTF-8", locale, &bytes_read, &bytes_written, &error);
						g_free(temporal);
					}
					g_print("%s",cadena);
				}
				g_free(cadena);
				temporal=NULL;
				cadena=NULL;
			}
		}
	} while (mas);	
	if (cadena!=NULL) {
		g_free(cadena);
	}
	salida=pclose(tuberia);
	DEBUG_MSG(<-gen_comando);
	return salida;
}

void gen_nuevo(GtkWidget *widget)
{
	GtkWidget *ventana;
	GtkEditable *entrada;
	gchar *nombre;
	gchar *archivo, *archivo_utf8;
	GtkTreeIter nodo_arbol;
	gchar *cabecera;
	gchar *defines;
	gchar *header;
	FILE *file;
	gint tipo;
	gint cont;
	gboolean todos_creados;
	gboolean guardar;
	gsize read, written;
	GError *error;
	
	DEBUG_MSG(->gen_nuevo);
	guardar=FALSE;
	ventana=gtk_widget_get_toplevel(widget);
	entrada=GTK_EDITABLE(glade_xml_get_widget(nuevo_ventana,"nuevo_entrada"));
	nombre=gtk_editable_get_chars(entrada,0,-1);
	
	do {
		todos_creados=TRUE;
		tipo=gen_tipo_archivo(nombre);
		if (!edit_repetido(nombre)) {
			DEBUG_MSG(el archivo no existe);
			archivo=pro_nombre_completo_archivo(nombre,FALSE, LOCALE);
			cabecera=pro_cabecera_gnu();
			if ((cabecera!=NULL) && (tipo!=0)) {
				file=fopen(archivo,"a+");
				fseek(file,0,SEEK_SET);
				fwrite(cabecera,1,strlen(cabecera),file);
				fclose(file);
				g_free(cabecera);
			}
			if ((tipo==2) && (pro_defines())) {
				header=g_strdup(nombre);
				for (cont=0;header[cont]!=0;cont++) {
					if ((header[cont]>='a') && (header[cont]<='z')) {
						header[cont]=header[cont]-32;
					}
					if (header[cont]=='.') {
						header[cont]='_';
					}
				}
				defines=g_strdup_printf("#ifndef %s\n#define %s\n\n\n\n#endif\n",header,header);
				file=fopen(archivo,"a+");
				fwrite(defines,1,strlen(defines),file);
				fclose(file);
				g_free(defines);
				g_free(header);
			}
			arch_anadir(nombre,&nodo_arbol);
			archivo_utf8 = g_locale_to_utf8(archivo, -1, &read, &written, &error);
			edit_anadir(archivo_utf8,&nodo_arbol);
			g_free(archivo_utf8);
			g_free(archivo);
			pro_anadir(nombre);
			guardar=TRUE;
			gen_cerrar_ventana(widget);
		} else {
			ventana=glade_xml_get_widget(repe_ventana,"repe_ventana");
			gtk_widget_show(ventana);
		}
		if ((pro_crear_h()) && (tipo==1)) {
			todos_creados=FALSE;
			for (cont=0;nombre[cont]!='.';cont++);
			nombre[cont+1]='h';
			nombre[cont+2]=0;
		}
	} while (!todos_creados);
	if (guardar) {
		pro_guardar();
	}
	g_free(nombre);
	DEBUG_MSG(<-gen_nuevo);
}

guint gen_tipo_archivo(gchar *nombre)
{
	// 0=otros, 1=fuentes, 2=cabeceras
	
	guint cont;
	
	for (cont=0;(nombre[cont]!='.') && (nombre[cont]!=0);cont++);
	if (nombre[cont]==0) {
		return 0;
	}
	cont++;
	if (!g_strcasecmp(nombre+cont,"c")) {
		return 1;
	}
	if (!g_strcasecmp(nombre+cont,"cpp")) {
		return 1;
	}
	if (!g_strcasecmp(nombre+cont,"c++")) {
		return 1;
	}
	if (!g_strcasecmp(nombre+cont,"cc")) {
		return 1;
	}
	if (!g_strcasecmp(nombre+cont,"h")) {
		return 2;
	}
	if (!g_strcasecmp(nombre+cont,"hh")) {
		return 2;
	}
	return 0;
}

void gen_ayuda(enum e_ayuda ayuda)
{
	GError *error;
	gchar *archivo;
	
	error=NULL;

	switch (ayuda) {
		case GTK:
			archivo=g_strdup(preferencias.ayuda_gtk);
			break;
		case XML:
			archivo=g_strdup(preferencias.ayuda_xml);
			break;
		case GLIB:
			archivo=g_strdup(preferencias.ayuda_glib);
			break;
		case ORBIT:
			archivo=g_strdup(preferencias.ayuda_orbit);
			break;
		case GNOME:
			archivo=g_strdup(preferencias.ayuda_gnome);
			break;
		case GNOMEUI:
			archivo=g_strdup(preferencias.ayuda_gnomeui);
			break;
		case GNORBA:
			archivo=g_strdup(preferencias.ayuda_gnorba);
			break;
	};

	if ((archivo!=NULL) && (archivo[0]!=0)) {
		gnome_help_display_uri(archivo,&error);
		if (error!=NULL) {
			g_printerr("%s\n",error->message);
			g_error_free(error);
		}
	}
}

void gen_eliminar(void)
{
	if (edit_actual!=NULL) {
		arch_eliminar(edit_actual->nombre);
		pro_eliminar(edit_actual->nombre);
		cefv_eliminar_archivo(edit_actual->nombre);
		edit_eliminar(edit_actual->nombre);
	}
}

void gen_cargar_proyecto(void)
{
	GtkWidget *combo;
	xmlNodePtr xmlnode;
	GtkTreeIter nodo_arbol;
	gchar *nombre;
	gchar *archivo;
	GtkTreeView *tree_view;
	gchar *glade_file, *name;
	gint retorno;
		
	DEBUG_MSG(->gen_cargar_proyecto);
	
	arch_crear();
	cefv_crear();
	
	edit_aplicar_reglas_coloreado(edit_funciones_buffer);
	
	xmlnode=NULL;
	do {
		nombre=pro_fuentes(&xmlnode, UTF8);
		if (nombre!=NULL) {
			archivo=pro_nombre_completo_archivo(nombre,FALSE, UTF8);
			arch_anadir(nombre,&nodo_arbol);
			edit_anadir(archivo,&nodo_arbol);
			g_free(nombre);
			g_free(archivo);
		}
	} while (nombre!=NULL);
	
	xmlnode=NULL;
	do {
		nombre=pro_cabeceras(&xmlnode, UTF8);
		if (nombre!=NULL) {
			archivo=pro_nombre_completo_archivo(nombre,FALSE, UTF8);
			arch_anadir(nombre,&nodo_arbol);
			edit_anadir(archivo,&nodo_arbol);
			g_free(nombre);
			g_free(archivo);
		}
	} while (nombre!=NULL);
	
	cefv_congelar_lista_combo=TRUE;
	xmlnode=NULL;
	do {
		nombre=pro_otros(&xmlnode, UTF8);
		if (nombre!=NULL) {
			archivo=pro_nombre_completo_archivo(nombre,FALSE, UTF8);
			arch_anadir(nombre,&nodo_arbol);
			edit_anadir(archivo,&nodo_arbol);
			g_free(nombre);
			g_free(archivo);
		}
	} while (nombre!=NULL);
	
	if (cefv_lista_combo!=NULL) {
		combo=glade_xml_get_widget(david_ventana,"funciones_combo");
		gtk_combo_set_popdown_strings(GTK_COMBO(combo),cefv_lista_combo);
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry),"");
		cefv_congelar_lista_combo=FALSE;
	}

	tree_view=GTK_TREE_VIEW(glade_xml_get_widget(david_ventana,"archivos_arbol"));
	gtk_tree_view_expand_all(tree_view);
	
	//Init the GUI tab treeview
	glade_window_list = NULL;
	name = pro_glade_file(UTF8);
	if (name != NULL)
		project_has_glade_file = TRUE;
	else
		project_has_glade_file = FALSE;
	if (project_has_glade_file == TRUE) {
		//parse the contents of the glade file
		glade_file = pro_nombre_completo_archivo(name, FALSE, UTF8);
		retorno = gla_create_glade_window_list(glade_file);
		if (retorno == 0)
			gla_gui_treeview_populate();

		glade_widget_list = NULL;
		glade_widget_list_last_widget = NULL;
		
		gla_parse_glade_project_file (glade_file);
		g_free(name);
		g_free(glade_file);
	}
	
	DEBUG_MSG(<-gen_cargar_proyecto);
}

void gen_recargar(void)
{
	typedef struct s_cargados {
		gchar *nombre;
		gint pestana;
		struct s_cargados *siguiente;	
	} s_cargados;
	
	struct s_lista_archivos *archivos;
	s_cargados *cargados,*ultimo;
	gchar *nombre_proyecto,*nombre_temp;
	gint ultima_pestana;
	GtkWidget *widget;

	DEBUG_MSG(->gen_recargar);
	if (proyecto.xml!=NULL) {
		nombre_proyecto=pro_nombre(UTF8);
		nombre_temp=g_strdup_printf("%s.diasceprj",nombre_proyecto);
		g_free(nombre_proyecto);
		nombre_proyecto=pro_nombre_completo_archivo(nombre_temp,FALSE, UTF8);
		g_free(nombre_temp);
		// primero creamos una lista con los archivos abiertos, y la pestaa en la que estan
		archivos=raiz_lista_archivos;
		cargados=NULL;
		ultimo=NULL;
		while (archivos!=NULL) {
			if (archivos->pestana!=-1) {
				if (cargados==NULL) {
					cargados=malloc(sizeof(s_cargados));
					ultimo=cargados;
				} else {
					ultimo->siguiente=malloc(sizeof(s_cargados));
					ultimo=ultimo->siguiente;
				}
				ultimo->nombre=g_strdup(archivos->nombre);
				ultimo->pestana=archivos->pestana;
				ultimo->siguiente=NULL;
			}
			archivos=archivos->siguiente;
		}
		
		// ahora recargamos el proyecto
		gen_abrir_proyecto(nombre_proyecto);
		g_free(nombre_proyecto);
		
		// ponemos cada archivo en su pestaa
		ultima_pestana=2;
		if (cargados!=NULL) {
			ultimo=cargados;
			while (ultimo!=NULL) {
				if (ultimo->pestana==ultima_pestana) {
					edit_ver(ultimo->nombre);
					on_mover_a_pestana_nueva1_activate(NULL,NULL);
					ultimo=cargados;
					ultima_pestana++;
				} else {
					ultimo=ultimo->siguiente;
				}
			}
			ultimo=cargados;
			while (ultimo!=NULL) {
				if (ultimo->pestana==0) {
					edit_ver(ultimo->nombre);
				}
				ultimo=ultimo->siguiente;
			}
		}
		widget=glade_xml_get_widget(david_ventana,"archivos_notebook");
		gtk_notebook_set_page(GTK_NOTEBOOK(widget),0);
		
		// eliminamos la lista creada
		while (cargados!=NULL) {
			g_free(cargados->nombre);
			ultimo=cargados;
			cargados=cargados->siguiente;
			g_free(ultimo);
		}
	}
	DEBUG_MSG(<-gen_recargar);
}

void gen_limpiar(void)
{
	GtkLabel *etiqueta;
	GtkWidget *notebook;
	gint cont;
	GtkCombo *historico_funciones;
		
	gen_limpiar_mensajes();
	DEBUG_MSG(->gen_limpiar);
	
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(edit_editor), empty_buffer);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(edit_editor),TRUE);
	gtk_text_buffer_set_text(GTK_TEXT_BUFFER(edit_funciones_buffer),"",-1);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(edit_editor),FALSE);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(edit_funciones),FALSE);
	notebook=glade_xml_get_widget(david_ventana,"archivos_notebook");
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook),0);
	for (cont=edit_total_pestanas;cont!=1;cont--) {
		gtk_notebook_remove_page(GTK_NOTEBOOK(notebook),cont);
	}
	edit_total_pestanas=1;
	historico_funciones=GTK_COMBO(glade_xml_get_widget(david_ventana,"historico_funciones_combo"));
	gtk_combo_set_popdown_strings(historico_funciones,NULL);
	gtk_entry_set_text(GTK_ENTRY(historico_funciones->entry),"");

	etiqueta=GTK_LABEL(glade_xml_get_widget(david_ventana,"editor_etiqueta"));
	gtk_label_set(etiqueta,_("Files"));
	etiqueta=GTK_LABEL(glade_xml_get_widget(david_ventana,"fila_etiqueta"));
	gtk_label_set(etiqueta,_("Row: 0  Colum:0"));
	edit_mensaje("");
	if (proyecto.xml!=NULL) {
		xmlFreeDoc(proyecto.xml);
		edit_cerrar();
		gla_free_glade_window_list();
		g_free(proyecto.archivo);
	}
	if (proyecto.xml_viejo!=NULL) {
		xmlFreeDoc(proyecto.xml_viejo);
	}
	pro_inicializar();
	edit_inicializar();
	buscar_inicializar();
	lista_cefv=NULL;
	DEBUG_MSG(<-gen_limpiar);
}

void gen_crear_proyecto(GtkWidget *widget)
{
	gen_limpiar();
	pro_crear();
	pro_actualizar(widget);
	arch_crear();
	cefv_crear();
}

void gen_abrir_proyecto(gchar *nombre)
{
	GtkWidget *ventana, *menu, *widget, *message_dialog;
	xmlNodePtr nodo,nodonuevo;
	gint cont;
	gchar *ultimo;
	gint retorno;
	
	DEBUG_MSG(->gen_abrir_proyecto);
	
	gen_limpiar();
	proyecto.archivo=g_strdup(nombre);
	retorno = pro_abrir();
	if (retorno == 0) {
		nodo=buscar_nodo_xml(preferencias.xml->xmlRootNode, "ultimos");
		nodonuevo=xmlNewNode(NULL,"ultimo");
		nodonuevo->parent=nodo;
		nodonuevo->next=nodo->xmlChildrenNode;
		nodo->xmlChildrenNode=nodonuevo;
		if (nodonuevo->next!=NULL) {
			nodonuevo->next->prev=nodonuevo;
		}
		xmlNodeSetContent(nodonuevo,proyecto.archivo);
		/*Creamos un menuitem para este ultimo proyecto en el menu File*/
		/*Si es el primero, habra que poner el separador tambien*/
		if (separador_recientes == FALSE) {
			widget = gtk_menu_item_new();
			menu = glade_xml_get_widget(david_ventana, "new2");
			menu = menu->parent;
			gtk_menu_insert(GTK_MENU(menu), widget, PRIMER_PROYECTO_RECIENTE);
			gtk_widget_show(widget);
			//Como el separador solo hay que ponerlo una vez, activamos un flag 
			separador_recientes = TRUE;		
		}
		menu = glade_xml_get_widget(david_ventana, "new2");
		menu = menu->parent;
		widget = gtk_menu_item_new_with_label (proyecto.archivo);
		gtk_menu_insert(GTK_MENU(menu), widget,PRIMER_PROYECTO_RECIENTE);
		/*Le asociamos la callback... sera la misma para todos */
		g_signal_connect(G_OBJECT(widget), "activate",G_CALLBACK(gen_recent_project_open), NULL);
		gtk_widget_show(widget);
		cont=1;
		nodo=nodo->xmlChildrenNode;
		while (nodo->next!=NULL) {
			nodo=nodo->next;
			ultimo=xmlNodeGetContent(nodo);
			if (!strcmp(ultimo,proyecto.archivo)) {
				/*El proyecto ya existia entre los ultimos abiertos, asi que eliminamos el nodo viejo*/
				nodonuevo=nodo;
				nodo=nodo->prev;
				xmlUnlinkNode(nodonuevo);
				xmlFreeNode(nodonuevo);
				/*Quitamos el menuitem correspondiente del menu File*/
				/*Obtenemos el menuitem*/
				gtk_menu_set_active(GTK_MENU(menu), PRIMER_PROYECTO_RECIENTE+cont);
				widget = gtk_menu_get_active(GTK_MENU(menu));
				/*y lo eliminamos*/
				gtk_widget_destroy(widget);
				cont--;
			}
			g_free(ultimo);
			cont++;
		}
		if (cont==6) {
			xmlUnlinkNode(nodo);
			xmlFreeNode(nodo);
			/*Quitamos el menuitem correspondiente del menu File*/
			/*Obtenemos el menuitem*/
			gtk_menu_set_active(GTK_MENU(menu), PRIMER_PROYECTO_RECIENTE+cont);
			widget = gtk_menu_get_active(GTK_MENU(menu));
			/*y lo eliminamos*/
			gtk_widget_destroy(widget);
		}
		pref_guardar();
		pref_xml_struct();
		gen_cargar_proyecto();
	} else {
		xmlFreeDoc(proyecto.xml);
		proyecto.xml=NULL;
		g_free(proyecto.archivo);
		proyecto.archivo=NULL;
		if (retorno == -1) {
			ventana = glade_xml_get_widget(david_ventana, "david_ventana");
			message_dialog = gtk_message_dialog_new(GTK_WINDOW(ventana), 
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_WARNING,
				GTK_BUTTONS_OK,
				"%s\n", _("You are trying to open a DiaSCE1 project file. DiaSCE2 does not support DiaSCE1 project files.\n You'll have to make a new DiaSCE2 project.")); 
			gtk_window_set_default_size(GTK_WINDOW(message_dialog), 200, 100);
			gtk_dialog_run (GTK_DIALOG(message_dialog));
			gtk_widget_destroy (message_dialog);
		} else {
			ventana=glade_xml_get_widget(no_proyecto_ventana,"no_proyecto_ventana");
			gtk_widget_show(ventana);
		}
	}
	DEBUG_MSG(<-gen_abrir_proyecto);
}

void gen_mensajes_stderr(gchar *mensaje)
{
	GtkTextView *mensajes;
	GtkTextIter start_iter, end_iter;
	GtkTextBuffer *buffer;
	
	mensajes=GTK_TEXT_VIEW(glade_xml_get_widget(david_ventana,"mensajes_texto"));
	buffer=gtk_text_view_get_buffer(mensajes);
	
	//we add the text at the end of the buffer always
	gtk_text_buffer_get_bounds(buffer, &start_iter, &end_iter);
	gtk_text_buffer_insert_with_tags_by_name(buffer, &end_iter, mensaje, -1,"mensaje_error",NULL);
}

void gen_mensajes_stdout(gchar *mensaje)
{
	GtkTextView *mensajes;
	GtkTextIter start_iter,end_iter;
	GtkTextBuffer *buffer;
	
	mensajes=GTK_TEXT_VIEW(glade_xml_get_widget(david_ventana,"mensajes_texto"));
	buffer=gtk_text_view_get_buffer(mensajes);

	//we add the text at the end of the buffer always
	gtk_text_buffer_get_bounds(buffer, &start_iter, &end_iter);
	gtk_text_buffer_insert(buffer, &end_iter, mensaje, -1);
	gtk_text_view_scroll_mark_onscreen(mensajes,gtk_text_buffer_get_insert(buffer));
}

void gen_limpiar_mensajes(void)
{
	GtkTextView *mensajes;
	GtkTextIter start_iter, end_iter;
	GtkTextBuffer *buffer;
	
	mensajes=GTK_TEXT_VIEW(glade_xml_get_widget(david_ventana,"mensajes_texto"));
	buffer=gtk_text_view_get_buffer(mensajes);
	gtk_text_buffer_get_bounds(buffer, &start_iter, &end_iter);
	gtk_text_buffer_delete(buffer, &start_iter, &end_iter);
}

gboolean gen_existe_fichero(gchar *nombre)
{
	FILE *archivo;
	gchar *nombre_locale;
	gsize read, written;
	GError *error;
	
	nombre_locale = g_locale_from_utf8(nombre, -1, &read, &written, &error);
	archivo=fopen(nombre_locale,"r");
	g_free(nombre_locale);
	if (archivo==NULL) {
		return FALSE;
	} else {
		fclose(archivo);
		return TRUE;
	}
}

void gen_anular_funciones(void)
{
	GtkWidget *widget;
	
	if (!ejecutables.lpr) {
		widget=glade_xml_get_widget(david_ventana,"imprimir_boton");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"imprimir1");
		gtk_widget_set_sensitive(widget,FALSE);
		g_printerr(_("lpr was not found in your path, you will need it to print with DiaSCE\n"));
	}
	if (!ejecutables.grep) {
		g_printerr(_("grep was not found in your path, you will need it to search in multiple files\n"));
	}
	if (!ejecutables.gcc) {
		widget=glade_xml_get_widget(david_ventana,"compilar_boton");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"compile1");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"make_boton");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"make1");
		gtk_widget_set_sensitive(widget,FALSE);
		g_printerr(_("gcc was not found in your path, you will need it to compile with DiaSCE\n"));
	}
	if (!ejecutables.gdb) {
		widget=glade_xml_get_widget(david_ventana,"ddd_button");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"ddd1");
		gtk_widget_set_sensitive(widget,FALSE);
		g_printerr(_("gdb was not found in your path, you will need it to debug with DiaSCE\n"));
	}
	if (!ejecutables.ddd) {
		widget=glade_xml_get_widget(david_ventana,"ddd_button");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"ddd1");
		gtk_widget_set_sensitive(widget,FALSE);
		g_printerr(_("ddd was not found in your path, you will need it to debug with DiaSCE\n"));
	}
	if (!ejecutables.glade) {
		widget=glade_xml_get_widget(david_ventana,"glade_boton");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"glade1");
		gtk_widget_set_sensitive(widget,FALSE);
		g_printerr(_("glade was not found in your path, you will nedd it do create GUIs with a graphic enviroment with DiaSCE\n"));
	}
	if (!ejecutables.ctags) {
		widget=glade_xml_get_widget(david_ventana,"abrir_boton");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"project1");
		gtk_widget_set_sensitive(widget,FALSE);
		//widget=glade_xml_get_widget(david_ventana,"abrir1");
		//gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"nuevo1");
		gtk_widget_set_sensitive(widget,FALSE);
		g_printerr(_("ctags was not found in your path, you will need it for DiaSCE to work properly\n"));
	}
	if (!ejecutables.memprof) {
		widget=glade_xml_get_widget(david_ventana,"memprof_boton");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"memprof2");
		gtk_widget_set_sensitive(widget,FALSE);
		g_printerr(_("MemProf was not found in your path\n"));
	}
	if (!ejecutables.gnome_terminal) {
		widget=glade_xml_get_widget(david_ventana,"ejecutar_boton");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"run1");
		gtk_widget_set_sensitive(widget,FALSE);
		g_printerr(_("GnomeTerminal was not found in your path, you will need it to run your executable from DiaSCE\n"));
	}
	if (!ejecutables.make) {
		widget=glade_xml_get_widget(david_ventana,"ejecutar_boton");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"run1");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"make_boton");
		gtk_widget_set_sensitive(widget,FALSE);
		widget=glade_xml_get_widget(david_ventana,"make1");
		gtk_widget_set_sensitive(widget,FALSE);
		g_printerr(_("make was not found in your path, you will need it to compile with DiaSCE\n"));
	}
}

/*****************************************************************************************************************
Esta funcion es la encargada de aadir en el menu "File" la lista con los ultimos proyectos abiertos,
para un acceso mas rapido a ellos
*****************************************************************************************************************/
void gen_ultimos_proyectos (void)
{
	xmlNodePtr nodo;
	gchar *recent;
	GtkWidget *widget, *menu;
	int i = 0;
	
	/* Nos posicionamos en el nodo "ultimos" del fichero .diasce2. Los children de este nodo contienen
	los ultimos proyectos abiertos*/
	nodo = buscar_nodo_xml (preferencias.xml->xmlRootNode, "ultimos");
	if (nodo != NULL) {
		/*nos situamos en el primer child*/
		nodo = nodo->xmlChildrenNode;
		/*si hay alguno, aadimos un separator primero*/
		if (nodo != NULL) {
			widget = gtk_menu_item_new();
			menu = glade_xml_get_widget(david_ventana, "new2");
			menu = menu->parent;
			gtk_menu_insert(GTK_MENU(menu), widget, PRIMER_PROYECTO_RECIENTE);
			gtk_widget_show(widget);
			//Como el separador solo hay que ponerlo una vez, activamos un flag 
			separador_recientes = TRUE;		
		} else {
			separador_recientes = FALSE;
		}
		while (nodo != NULL) {
			recent = xmlNodeGetContent (nodo);
			/*Creamos un nuevo menuitem, cuya label es el nombre de uno de los ultimos proyectos abiertos*/
			widget = gtk_menu_item_new_with_label (recent);
			gtk_menu_insert(GTK_MENU(menu), widget,PRIMER_PROYECTO_RECIENTE+i);
			/*Le asociamos la callback... sera la misma para todos */
			g_signal_connect(G_OBJECT(widget), "activate",G_CALLBACK(gen_recent_project_open), NULL);
			gtk_widget_show(widget);
			g_free(recent);
			nodo = nodo->next;
			i = i+1;
		}
	}
}

/*****************************************************************************************
Funcion encargada de abrir un proyecto reciente. Esta asociada a los menuitems
de ultimos proyectos, en el menu File. Recoge el label del menuitem que le ha llamado
en user_data y abre el proyecto indicado por ese label.
*****************************************************************************************/
void gen_recent_project_open (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *ventana;
	gchar *archivo;
	gint ret;
	
	DEBUG_MSG(->gen_recent_project_open);
	
	if (GTK_BIN(menuitem)->child) {
		GtkWidget *child = GTK_BIN(menuitem)->child;
		if (GTK_IS_LABEL(child)) {
			const gchar *text;
		
			text=gtk_label_get_text(GTK_LABEL(child));
			archivo = g_strdup (text);	
			g_print("%s\n",archivo);
		}
		if (edit_comprobar_perdida_datos()) {
			g_free(pro_proyecto_cargar);
			pro_proyecto_cargar=g_strdup(archivo);
			ventana = gen_create_perdidas_ventana();
			ret = gnome_dialog_run_and_close (GNOME_DIALOG(ventana));
			switch (ret) {
				case 0: //Boton "si"
					gen_abrir_proyecto(archivo);
					break;
				case 1: //Boton "no"
					break;
				default:	//El usuario ha cerrado la ventana desde el gestor de ventanas
					break;
			}
			//Creo que no hace falta
			//gtk_widget_hide(ventana); 
		} else {
			gen_abrir_proyecto(archivo);
		}
		g_free (archivo);
	}
	DEBUG_MSG(<-gen_recent_project_open);
}

gboolean gen_es_salida_grep(gchar *salida)
{
	int cont,dos_puntos,cont1,cont2;
	gboolean es_salida_grep;
	
	cont=0;
	dos_puntos=0;
	while (salida[cont]!='\n') {
		if (salida[cont]==':') {
			dos_puntos++;
			if (dos_puntos==1) cont1=cont;
			if (dos_puntos==2) cont2=cont;
		}
		cont++;
	}

	es_salida_grep=FALSE;
	if (dos_puntos==2) {
		es_salida_grep=TRUE;
		for (cont=cont1+1;cont<cont2;cont++) {
			if ((salida[cont]<'0') || (salida[cont]>'9')) es_salida_grep=FALSE;
		if (salida[cont2+1]=='\n') es_salida_grep=FALSE;
		}
	};
	return es_salida_grep;
}

/**********************************************************************************************
* Esta funcion crea la ventana que indica al usuario que hay ficheros modificados   *
* sin guardar y que si continua se perderan esos datos. Muestra la lista de ficheros  *
* modificados que no han sido guardados, y devuelve un puntero a la ventana        *
* que luego podra ser mostrada con una llamada a gnome_run_dialog                     *
***********************************************************************************************/
GtkWidget *gen_create_perdidas_ventana (void)
{
	GtkWidget *widget;
	struct s_lista_archivos *nodo;
	gchar *archivos_modificados, *tmp, *mensaje;

	//Primero creamos una cadena que contenga una lista de los archivos modificados
	//que no han sido guardados
	archivos_modificados = g_strdup("");
	nodo = raiz_lista_archivos;
	while (nodo != NULL) {
		if (gtk_text_buffer_get_modified(GTK_TEXT_BUFFER(nodo->datos))) {
			tmp = g_strdup(archivos_modificados);
			g_free(archivos_modificados);
			archivos_modificados = g_strconcat (tmp,nodo->nombre,"\n",NULL);
			g_free(tmp);
		}
		nodo = nodo->siguiente;
	}
	//Generamos el mensaje
	mensaje = g_strconcat(
		_("Warning! The following files have unsaved changes:\n\n"), 
		archivos_modificados,
		_("\nIf you continue, data will be lost.\n\nDo you still want to continue?\n"),
		NULL); 
	//una vez que tenemos la lista, generamos la ventana con el mensaje:
	widget = gnome_message_box_new(
		mensaje, 
		GNOME_MESSAGE_BOX_WARNING,
		GNOME_STOCK_BUTTON_YES,
		GNOME_STOCK_BUTTON_NO,
		NULL);
	g_free(archivos_modificados);
	g_free(mensaje);
	//y retornamos un puntero a la ventana
	return widget;
}

/**********************************************************************************
 * esta funcion hace que se atiendan a los eventos pendientes antes de *
 * devolver el control a gtk_main(). De esta forma se puede refrescar la   *
 * pantalla en sitios donde se retenga el control de programa durante      *
 * mucho timepo, como durante la compilacion.                                     *
 **********************************************************************************/
void gen_refrescar(void)
{
	while (gtk_events_pending()) {
		gtk_main_iteration();
	}
}

/**********************************************************************************
 esta funcion sera colgada de la seal CHILD para que haga un wait
 cada vez que un hijo muera. De esta forma se evita que aparezcan
 como "zombi"
 **********************************************************************************/
void gen_hijo_muerto(void)
{
	wait(NULL);
	signal(SIGCHLD,(void *)gen_hijo_muerto);
}

/********************************************************************************************
This function creates the GUI of DiaSCE using Libglade. The code editor widget and
the function editor widget are added manually because we tryed putting them in glade
and they did not work correctly
*********************************************************************************/
gint gen_create_david_gui(void)
{
	GtkWidget *editor_texto, *funciones_texto, *mensajes_texto;
	GtkWidget *scrolledwindow;
	GtkWidget *funciones_vbox, *editor_vbox;
	GtkTextBuffer *buffer;
	FILE *fichero;
	
	fichero = fopen (GLADE_PROJECT, "r");
	if (fichero == NULL) {
		return -1;
	} else {
		fclose(fichero);
	}
	
	david_ventana = glade_xml_new(GLADE_PROJECT,"david_ventana",NULL);
	glade_xml_signal_autoconnect(david_ventana);
	
	proyecto_ventana = glade_xml_new(GLADE_PROJECT,"proyecto_ventana",NULL);
	glade_xml_signal_autoconnect(proyecto_ventana);
	
	acercade_ventana = glade_xml_new(GLADE_PROJECT,"acercade_ventana",NULL);
	glade_xml_signal_autoconnect(acercade_ventana);
	
	preferencias_ventana = glade_xml_new(GLADE_PROJECT,"preferencias_ventana",NULL);
	glade_xml_signal_autoconnect(preferencias_ventana);
	
	archivos_arbol_menu = glade_xml_new(GLADE_PROJECT,"archivos_arbol_menu",NULL);
	glade_xml_signal_autoconnect(archivos_arbol_menu);

	editor_menu = glade_xml_new(GLADE_PROJECT,"editor_menu",NULL);
	glade_xml_signal_autoconnect(editor_menu);

	buscar_ventana = glade_xml_new(GLADE_PROJECT,"buscar_ventana",NULL);
	glade_xml_signal_autoconnect(buscar_ventana);

	nuevo_ventana = glade_xml_new(GLADE_PROJECT,"nuevo_ventana",NULL);
	glade_xml_signal_autoconnect(nuevo_ventana);

	repe_ventana = glade_xml_new(GLADE_PROJECT,"repe_ventana",NULL);
	glade_xml_signal_autoconnect(repe_ventana);

	no_proyecto_ventana = glade_xml_new(GLADE_PROJECT,"no_proyecto_ventana",NULL);
	glade_xml_signal_autoconnect(no_proyecto_ventana);

	borrar_archivo_ventana = glade_xml_new(GLADE_PROJECT,"borrar_archivo_ventana",NULL);
	glade_xml_signal_autoconnect(borrar_archivo_ventana);

	autodetect_ventana = glade_xml_new(GLADE_PROJECT,"autodetect_ventana",NULL);
	glade_xml_signal_autoconnect(autodetect_ventana);
	
	editor_vbox = glade_xml_get_widget(david_ventana,"editor_notebook_vbox");
	editor_texto = editor_texto_new(NULL,NULL,NULL,0,0);
	g_signal_connect (G_OBJECT (editor_texto), "enter_notify_event",G_CALLBACK(on_editor_texto_enter_notify_event),NULL);
	g_signal_connect (G_OBJECT (editor_texto), "button_press_event",G_CALLBACK(on_editor_texto_button_press_event),NULL);
	g_signal_connect (G_OBJECT (editor_texto), "key_press_event",G_CALLBACK(on_editor_texto_key_press_event),NULL);
	g_signal_connect_after (G_OBJECT (editor_texto), "key_press_event",G_CALLBACK(on_editor_texto_key_press_event_despues),NULL);
	scrolledwindow=gtk_scrolled_window_new(NULL,NULL);
	gtk_container_add(GTK_CONTAINER(scrolledwindow),editor_texto);
	gtk_source_view_set_tab_stop(GTK_SOURCE_VIEW(editor_texto),preferencias.tabulador);
	gtk_container_add(GTK_CONTAINER(editor_vbox),scrolledwindow);
	gtk_widget_show(editor_texto);
	gtk_widget_show(scrolledwindow);
	//global variable that refers to the editor of the File Tab
	edit_editor = GTK_SOURCE_VIEW(editor_texto);
	
	funciones_vbox = glade_xml_get_widget(david_ventana,"funciones_notebook_vbox");
	funciones_texto = editor_texto_new(NULL,NULL,NULL,0,0);
	g_signal_connect (G_OBJECT (funciones_texto), "button_press_event",G_CALLBACK(on_editor_texto_button_press_event),NULL);
	scrolledwindow=gtk_scrolled_window_new(NULL,NULL);
	gtk_container_add(GTK_CONTAINER(scrolledwindow),funciones_texto);
	gtk_source_view_set_tab_stop(GTK_SOURCE_VIEW(funciones_texto),preferencias.tabulador);
	gtk_container_add(GTK_CONTAINER(funciones_vbox),scrolledwindow);
	gtk_widget_show(funciones_texto);
	gtk_widget_show(scrolledwindow);
	//global variable that refers to the editor of the Function Tab
	edit_funciones = GTK_SOURCE_VIEW(funciones_texto);

	//mark_set callback for the message window
	mensajes_texto = glade_xml_get_widget (david_ventana, "mensajes_texto");
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(mensajes_texto));
	g_signal_connect_after (G_OBJECT(buffer), "mark_set", G_CALLBACK(on_mensajes_texto_buffer_mark_set), NULL);

	return 0;
}

gint gen_GtkTreeIterCompareFunc(GtkTreeModel *model,GtkTreeIter *a,GtkTreeIter *b,gpointer user_data)
{
	gchar *texto_a,*texto_b;
	gint comparacion;

	if (model==GTK_TREE_MODEL(arch_store)) {
		if ((a->stamp==arch_fuentes.stamp) || (a->stamp==arch_cabeceras.stamp) || (a->stamp==arch_otros.stamp)) {
			return 0;
		}
	}
	
	gtk_tree_model_get(model,a,COL_NOMBRE,&texto_a,-1);
	gtk_tree_model_get(model,b,COL_NOMBRE,&texto_b,-1);
	
	if ((texto_a!=NULL) && (texto_b!=NULL)) {
		comparacion=strcasecmp(texto_a,texto_b);
	} else {
		comparacion=0;
	}
	
	g_free(texto_a);
	g_free(texto_b);
	
	return comparacion;
}

void gen_localizar_automake(void)
{	
	FILE *tuberia;
	gchar letra;
	gchar *cadena;
	gchar *temporal;
	guint mas;
	gchar *prefix_end;

	DEBUG_MSG(->gen_localizar_automake);
	if ((!ejecutables.which) || (!ejecutables.automake)) {
		prefix_automake=g_strdup("/usr");
		DEBUG_MSG(<-gen_localizar_automake 1);
		return;
	}

	tuberia=popen("which automake","r");
	temporal=NULL;
	cadena=NULL;
	do {
		mas=fread(&letra,1,1,tuberia);
		if (mas) {
			if (temporal!=NULL) {
				cadena=g_strdup_printf("%s%c",temporal,letra);
				g_free(temporal);
				temporal=cadena;
			} else {
				cadena=g_strdup_printf("%c",letra);
				temporal=cadena;
			}
		}
	} while (mas);
	
	prefix_end=strstr(cadena,"/bin/automake");
	if (prefix_end!=NULL) {
		prefix_end[0]=0;
		prefix_automake=g_strdup(cadena);
	} else{
		prefix_automake=g_strdup("/usr");
	}
	
	if (cadena!=NULL) {
		g_free(cadena);
	}
	pclose(tuberia);
	DEBUG_MSG(<-gen_localizar_automake);
}
